// Copyright 2023 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// GENERATED FROM THE API DEFINITION IN
//   extensions/common/api/content_scripts.idl
// by tools/json_schema_compiler.
// DO NOT EDIT.

#include "extensions/common/api/content_scripts.h"

#include <memory>
#include <ostream>
#include <string>
#include <utility>
#include <vector>

#include "base/check.h"
#include "base/check_op.h"
#include "base/notreached.h"
#include "base/strings/string_number_conversions.h"
#include "base/strings/utf_string_conversions.h"
#include "base/values.h"
#include "tools/json_schema_compiler/util.h"
#include "tools/json_schema_compiler/manifest_parse_util.h"

#include "base/strings/string_piece.h"
#include "extensions/common/api/extension_types.h"


using base::UTF8ToUTF16;

namespace extensions {
namespace api {
namespace content_scripts {
//
// Types
//

const char* ToString(RunAt enum_param) {
  switch (enum_param) {
    case RUN_AT_DOCUMENT_IDLE:
      return "document_idle";
    case RUN_AT_DOCUMENT_START:
      return "document_start";
    case RUN_AT_DOCUMENT_END:
      return "document_end";
    case RUN_AT_NONE:
      return "";
  }
  NOTREACHED();
  return "";
}

RunAt ParseRunAt(const std::string& enum_string) {
  if (enum_string == "document_idle")
    return RUN_AT_DOCUMENT_IDLE;
  if (enum_string == "document_start")
    return RUN_AT_DOCUMENT_START;
  if (enum_string == "document_end")
    return RUN_AT_DOCUMENT_END;
  return RUN_AT_NONE;
}


ContentScript::ContentScript()
: run_at(RUN_AT_NONE),
world(extension_types::EXECUTION_WORLD_NONE) {}

ContentScript::~ContentScript() = default;
ContentScript::ContentScript(ContentScript&& rhs) = default;
ContentScript& ContentScript::operator=(ContentScript&& rhs) = default;
// static
constexpr char ContentScript::kMatches[];
// static
constexpr char ContentScript::kExcludeMatches[];
// static
constexpr char ContentScript::kCss[];
// static
constexpr char ContentScript::kJs[];
// static
constexpr char ContentScript::kAllFrames[];
// static
constexpr char ContentScript::kMatchOriginAsFallback[];
// static
constexpr char ContentScript::kMatchAboutBlank[];
// static
constexpr char ContentScript::kIncludeGlobs[];
// static
constexpr char ContentScript::kExcludeGlobs[];
// static
constexpr char ContentScript::kRunAt[];
// static
constexpr char ContentScript::kWorld[];

// static
bool ContentScript::Populate(
    const base::Value& value, ContentScript* out, std::u16string* error) {
  DCHECK(error);
  if (!value.is_dict()) {
    DCHECK(error->empty());
    *error = u"expected dictionary, got " + UTF8ToUTF16(base::Value::GetTypeName(value.type()));
    return false;
  }
  const auto* dict = static_cast<const base::DictionaryValue*>(&value);
  out->run_at = RUN_AT_NONE;
  out->world = extension_types::EXECUTION_WORLD_NONE;
  const base::Value* matches_value = dict->FindKey("matches");
  if (!matches_value) {
    DCHECK(error->empty());
    *error = u"'matches' is required";
    return false;
  }
  {
    if (!(*matches_value).is_list()) {
      DCHECK(error->empty());
      *error = u"'matches': expected list, got " + UTF8ToUTF16(base::Value::GetTypeName((*matches_value).type()));
      return false;
    }
    else {
      std::u16string array_parse_error;
      if (!json_schema_compiler::util::PopulateArrayFromList((*matches_value).GetListDeprecated(), &out->matches, &array_parse_error)) {
        array_parse_error = u"Error at key 'matches': " + array_parse_error;
        DCHECK(error->empty());
        *error = array_parse_error;
        return false;
      }
    }
  }

  const base::Value* exclude_matches_value = dict->FindKey("exclude_matches");
  if (exclude_matches_value) {
    {
      if (!(*exclude_matches_value).is_list()) {
        DCHECK(error->empty());
        *error = u"'exclude_matches': expected list, got " + UTF8ToUTF16(base::Value::GetTypeName((*exclude_matches_value).type()));
        return false;
      }
      else {
        std::u16string array_parse_error;
        if (!json_schema_compiler::util::PopulateOptionalArrayFromList((*exclude_matches_value).GetListDeprecated(), &out->exclude_matches, &array_parse_error)) {
          array_parse_error = u"Error at key 'exclude_matches': " + array_parse_error;
          DCHECK(error->empty());
          *error = array_parse_error;
          return false;
        }
      }
    }
  }

  const base::Value* css_value = dict->FindKey("css");
  if (css_value) {
    {
      if (!(*css_value).is_list()) {
        DCHECK(error->empty());
        *error = u"'css': expected list, got " + UTF8ToUTF16(base::Value::GetTypeName((*css_value).type()));
        return false;
      }
      else {
        std::u16string array_parse_error;
        if (!json_schema_compiler::util::PopulateOptionalArrayFromList((*css_value).GetListDeprecated(), &out->css, &array_parse_error)) {
          array_parse_error = u"Error at key 'css': " + array_parse_error;
          DCHECK(error->empty());
          *error = array_parse_error;
          return false;
        }
      }
    }
  }

  const base::Value* js_value = dict->FindKey("js");
  if (js_value) {
    {
      if (!(*js_value).is_list()) {
        DCHECK(error->empty());
        *error = u"'js': expected list, got " + UTF8ToUTF16(base::Value::GetTypeName((*js_value).type()));
        return false;
      }
      else {
        std::u16string array_parse_error;
        if (!json_schema_compiler::util::PopulateOptionalArrayFromList((*js_value).GetListDeprecated(), &out->js, &array_parse_error)) {
          array_parse_error = u"Error at key 'js': " + array_parse_error;
          DCHECK(error->empty());
          *error = array_parse_error;
          return false;
        }
      }
    }
  }

  const base::Value* all_frames_value = dict->FindKey("all_frames");
  if (all_frames_value) {
    {
      auto temp = (*all_frames_value).GetIfBool();
      if (!temp.has_value()) {
        DCHECK(error->empty());
        *error = u"'all_frames': expected all_frames, got " + UTF8ToUTF16(base::Value::GetTypeName((*all_frames_value).type()));
        out->all_frames.reset();
        return false;
      }
      out->all_frames = std::make_unique<bool>(temp.value());
    }
  }

  const base::Value* match_origin_as_fallback_value = dict->FindKey("match_origin_as_fallback");
  if (match_origin_as_fallback_value) {
    {
      auto temp = (*match_origin_as_fallback_value).GetIfBool();
      if (!temp.has_value()) {
        DCHECK(error->empty());
        *error = u"'match_origin_as_fallback': expected match_origin_as_fallback, got " + UTF8ToUTF16(base::Value::GetTypeName((*match_origin_as_fallback_value).type()));
        out->match_origin_as_fallback.reset();
        return false;
      }
      out->match_origin_as_fallback = std::make_unique<bool>(temp.value());
    }
  }

  const base::Value* match_about_blank_value = dict->FindKey("match_about_blank");
  if (match_about_blank_value) {
    {
      auto temp = (*match_about_blank_value).GetIfBool();
      if (!temp.has_value()) {
        DCHECK(error->empty());
        *error = u"'match_about_blank': expected match_about_blank, got " + UTF8ToUTF16(base::Value::GetTypeName((*match_about_blank_value).type()));
        out->match_about_blank.reset();
        return false;
      }
      out->match_about_blank = std::make_unique<bool>(temp.value());
    }
  }

  const base::Value* include_globs_value = dict->FindKey("include_globs");
  if (include_globs_value) {
    {
      if (!(*include_globs_value).is_list()) {
        DCHECK(error->empty());
        *error = u"'include_globs': expected list, got " + UTF8ToUTF16(base::Value::GetTypeName((*include_globs_value).type()));
        return false;
      }
      else {
        std::u16string array_parse_error;
        if (!json_schema_compiler::util::PopulateOptionalArrayFromList((*include_globs_value).GetListDeprecated(), &out->include_globs, &array_parse_error)) {
          array_parse_error = u"Error at key 'include_globs': " + array_parse_error;
          DCHECK(error->empty());
          *error = array_parse_error;
          return false;
        }
      }
    }
  }

  const base::Value* exclude_globs_value = dict->FindKey("exclude_globs");
  if (exclude_globs_value) {
    {
      if (!(*exclude_globs_value).is_list()) {
        DCHECK(error->empty());
        *error = u"'exclude_globs': expected list, got " + UTF8ToUTF16(base::Value::GetTypeName((*exclude_globs_value).type()));
        return false;
      }
      else {
        std::u16string array_parse_error;
        if (!json_schema_compiler::util::PopulateOptionalArrayFromList((*exclude_globs_value).GetListDeprecated(), &out->exclude_globs, &array_parse_error)) {
          array_parse_error = u"Error at key 'exclude_globs': " + array_parse_error;
          DCHECK(error->empty());
          *error = array_parse_error;
          return false;
        }
      }
    }
  }

  const base::Value* run_at_value = dict->FindKey("run_at");
  if (run_at_value) {
    {
      const std::string* run_at_as_string = (*run_at_value).GetIfString();
      if (!run_at_as_string) {
        DCHECK(error->empty());
        *error = u"'RunAt': expected string, got " + UTF8ToUTF16(base::Value::GetTypeName((*run_at_value).type()));
        return false;
      }
      out->run_at = ParseRunAt(*run_at_as_string);
      if (out->run_at == RUN_AT_NONE) {
        DCHECK(error->empty());
        *error = u"'RunAt': expected \"document_idle\" or \"document_start\" or \"document_end\", got \"" + UTF8ToUTF16(*run_at_as_string) + u"\"";
        return false;
      }
    }
    } else {
    out->run_at = RUN_AT_NONE;
  }

  const base::Value* world_value = dict->FindKey("world");
  if (world_value) {
    {
      const std::string* execution_world_as_string = (*world_value).GetIfString();
      if (!execution_world_as_string) {
        DCHECK(error->empty());
        *error = u"'ExecutionWorld': expected string, got " + UTF8ToUTF16(base::Value::GetTypeName((*world_value).type()));
        return false;
      }
      out->world = extension_types::ParseExecutionWorld(*execution_world_as_string);
      if (out->world == extension_types::EXECUTION_WORLD_NONE) {
        DCHECK(error->empty());
        *error = u"'ExecutionWorld': expected \"ISOLATED\" or \"MAIN\", got \"" + UTF8ToUTF16(*execution_world_as_string) + u"\"";
        return false;
      }
    }
    } else {
    out->world = extension_types::EXECUTION_WORLD_NONE;
  }

  return true;
}

// static
std::unique_ptr<ContentScript> ContentScript::FromValue(const base::Value& value, std::u16string* error) {
  DCHECK(error);
  auto out = std::make_unique<ContentScript>();
  bool result = Populate(value, out.get(), error);
  DCHECK_EQ(result, error->empty());
  if (!result)
    return nullptr;
  return out;
}

std::unique_ptr<base::DictionaryValue> ContentScript::ToValue() const {
  auto to_value_result =
      std::make_unique<base::DictionaryValue>();

  to_value_result->SetWithoutPathExpansion("matches", json_schema_compiler::util::CreateValueFromArray(this->matches));

  if (this->exclude_matches.get()) {
    to_value_result->SetWithoutPathExpansion("exclude_matches", json_schema_compiler::util::CreateValueFromArray(*this->exclude_matches));

  }
  if (this->css.get()) {
    to_value_result->SetWithoutPathExpansion("css", json_schema_compiler::util::CreateValueFromArray(*this->css));

  }
  if (this->js.get()) {
    to_value_result->SetWithoutPathExpansion("js", json_schema_compiler::util::CreateValueFromArray(*this->js));

  }
  if (this->all_frames.get()) {
    to_value_result->SetWithoutPathExpansion("all_frames", std::make_unique<base::Value>(*this->all_frames));

  }
  if (this->match_origin_as_fallback.get()) {
    to_value_result->SetWithoutPathExpansion("match_origin_as_fallback", std::make_unique<base::Value>(*this->match_origin_as_fallback));

  }
  if (this->match_about_blank.get()) {
    to_value_result->SetWithoutPathExpansion("match_about_blank", std::make_unique<base::Value>(*this->match_about_blank));

  }
  if (this->include_globs.get()) {
    to_value_result->SetWithoutPathExpansion("include_globs", json_schema_compiler::util::CreateValueFromArray(*this->include_globs));

  }
  if (this->exclude_globs.get()) {
    to_value_result->SetWithoutPathExpansion("exclude_globs", json_schema_compiler::util::CreateValueFromArray(*this->exclude_globs));

  }
  if (this->run_at != RUN_AT_NONE) {
    to_value_result->SetWithoutPathExpansion("run_at", std::make_unique<base::Value>(content_scripts::ToString(this->run_at)));

  }
  if (this->world != extension_types::EXECUTION_WORLD_NONE) {
    to_value_result->SetWithoutPathExpansion("world", std::make_unique<base::Value>(extension_types::ToString(this->world)));

  }

  return to_value_result;
}

//static
bool ContentScript::ParseFromDictionary(
const base::DictionaryValue& root_dict, base::StringPiece key, ContentScript* out, std::u16string* error, std::vector<base::StringPiece>* error_path_reversed) {
  DCHECK(out);
  DCHECK(error);
  DCHECK(error_path_reversed);

  const base::Value* value = ::json_schema_compiler::manifest_parse_util::FindKeyOfType(root_dict, key, base::Value::Type::DICTIONARY, error, error_path_reversed);
  if (!value)
    return false;
  const base::DictionaryValue& dict = base::Value::AsDictionaryValue(*value);
  out->run_at = RUN_AT_NONE;
  out->world = extension_types::EXECUTION_WORLD_NONE;
  if (!::json_schema_compiler::manifest_parse_util::ParseFromDictionary(dict, kMatches, &out->matches, error, error_path_reversed)) {
    error_path_reversed->push_back(key);
    return false;
  }

  if (!::json_schema_compiler::manifest_parse_util::ParseFromDictionary(dict, kExcludeMatches, &out->exclude_matches, error, error_path_reversed)) {
    error_path_reversed->push_back(key);
    return false;
  }

  if (!::json_schema_compiler::manifest_parse_util::ParseFromDictionary(dict, kCss, &out->css, error, error_path_reversed)) {
    error_path_reversed->push_back(key);
    return false;
  }

  if (!::json_schema_compiler::manifest_parse_util::ParseFromDictionary(dict, kJs, &out->js, error, error_path_reversed)) {
    error_path_reversed->push_back(key);
    return false;
  }

  if (!::json_schema_compiler::manifest_parse_util::ParseFromDictionary(dict, kAllFrames, &out->all_frames, error, error_path_reversed)) {
    error_path_reversed->push_back(key);
    return false;
  }

  if (!::json_schema_compiler::manifest_parse_util::ParseFromDictionary(dict, kMatchOriginAsFallback, &out->match_origin_as_fallback, error, error_path_reversed)) {
    error_path_reversed->push_back(key);
    return false;
  }

  if (!::json_schema_compiler::manifest_parse_util::ParseFromDictionary(dict, kMatchAboutBlank, &out->match_about_blank, error, error_path_reversed)) {
    error_path_reversed->push_back(key);
    return false;
  }

  if (!::json_schema_compiler::manifest_parse_util::ParseFromDictionary(dict, kIncludeGlobs, &out->include_globs, error, error_path_reversed)) {
    error_path_reversed->push_back(key);
    return false;
  }

  if (!::json_schema_compiler::manifest_parse_util::ParseFromDictionary(dict, kExcludeGlobs, &out->exclude_globs, error, error_path_reversed)) {
    error_path_reversed->push_back(key);
    return false;
  }

  if (!::json_schema_compiler::manifest_parse_util::ParseEnumFromDictionary(dict, kRunAt, &ParseRunAt, true, RUN_AT_NONE, &out->run_at, error, error_path_reversed)) {
    error_path_reversed->push_back(key);
    return false;
  }

  if (!::json_schema_compiler::manifest_parse_util::ParseEnumFromDictionary(dict, kWorld, &extension_types::ParseExecutionWorld, true, extension_types::EXECUTION_WORLD_NONE, &out->world, error, error_path_reversed)) {
    error_path_reversed->push_back(key);
    return false;
  }

  return true;
}



//
// Manifest Keys
//

ManifestKeys::ManifestKeys()
 {}

ManifestKeys::~ManifestKeys() = default;
ManifestKeys::ManifestKeys(ManifestKeys&& rhs) = default;
ManifestKeys& ManifestKeys::operator=(ManifestKeys&& rhs) = default;
// static
constexpr char ManifestKeys::kContentScripts[];

//static
bool ManifestKeys::ParseFromDictionary(
const base::DictionaryValue& root_dict, ManifestKeys* out, std::u16string* error) {
  DCHECK(out);
  DCHECK(error);

  std::vector<base::StringPiece> error_path_reversed_vec;
  auto* error_path_reversed = &error_path_reversed_vec;
  const base::DictionaryValue& dict = root_dict;
  if (!::json_schema_compiler::manifest_parse_util::ParseFromDictionary(dict, kContentScripts, &out->content_scripts, error, error_path_reversed)) {
    ::json_schema_compiler::manifest_parse_util::PopulateFinalError(error, error_path_reversed);
    return false;
  }

  return true;
}


}  // namespace content_scripts
}  // namespace api
}  // namespace extensions

